/*
 * BoostController2.cpp
 *
 *  Created on: 08.02.2012
 *      Author: bofh
 */

#include "BoostController2.h"
#include "SensorData.h"
#include "PID_v1.h"
#include "BoostControllerDefines.h"
#include "EEPROM.h"

BoostController2::BoostController2() {
}

void BoostController2::myconstructor() {

	loadFromEeprom();

	boostOutput = 0;

	pid = new PID( (double*) &data.calBoost, &pidBoostOutput, &pidBoostSetPoint, aKp, aKi, aKd, DIRECT);

	//default sample time is 100ms
	pid->SetSampleTime(200);
	pid->SetMode(AUTOMATIC);

	pidBoostSetPoint = 0;
	pidBoostOutput = 0;

	req_Boost_PWM = 0;
	req_Boost = 0;

}

void BoostController2::loadFromEeprom () {
	n75_pwm = EEPROM.read (EEPROM_N75_PWM);
	uint16_t t2 = EEPROMReaduint16(EEPROM_N75_BOOST);
	if ( t2 < 0xFFFF )
		n75_boost = fixedintb1002float(t2);
	else
		n75_boost = 1.0;

	//equal to MD code
	uint16_t raw = 0;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_aKp);
	if ( raw < 0xFFFF )
		aKp = fixedintb1002float(raw);
	else
		aKp = 2;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_aKd);
	if ( raw < 0xFFFF )
		aKd = fixedintb1002float(raw);
	else
		aKd = 1;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_aKi);
	if ( raw < 0xFFFF )
		aKi = fixedintb1002float(raw);
	else
		aKi = 5;

	raw = EEPROMReaduint16 (EEPROM_N75_PID_cKp);
	if ( raw < 0xFFFF )
		cKp = fixedintb1002float(raw);
	else
		cKp = 1;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_cKd);
	if ( raw < 0xFFFF )
		cKd = fixedintb1002float(raw);
	else
		cKd = 0.5;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_cKi);
	if ( raw < 0xFFFF )
		cKi = fixedintb1002float(raw);
	else
		cKi = 3;

	raw = EEPROMReaduint16 (EEPROM_N75_PID_aAT);
	if ( raw < 0xFFFF )
		apidActivationThresholdFactor = fixedintb1002float(raw);
	else
		apidActivationThresholdFactor = 0.5;
	raw = EEPROMReaduint16 (EEPROM_N75_PID_cAT);
	if ( raw < 0xFFFF )
		cpidActivationThresholdFactor = fixedintb1002float(raw);
	else
		cpidActivationThresholdFactor = 0.85;

	uint8_t b = EEPROM.read(EEPROM_N75_ENABLE_PID);
	if (b==0 || b==255)
		usePID = false;
	else
		usePID = true;

	t2 = EEPROMReaduint16(EEPROM_N75_MAX_BOOST);
	if ( t2 < 0xFFFF )
		n75_max_boost = fixedintb1002float(t2);
	else
		n75_max_boost = 1.1;
}


void BoostController2::compute () {

	req_Boost_PWM = n75_pwm;
	req_Boost = n75_boost;
//		boostOutput = n75_high_boost;

	if ( usePID ) {
		//give the PID the requested boost level
		pidBoostSetPoint = req_Boost;
		//activate the PID only if a stable boost is reached
		double aat = pidBoostSetPoint * apidActivationThresholdFactor;
		double cat = pidBoostSetPoint * cpidActivationThresholdFactor;

		if ( data.calBoost > aat  ) {
			if ( data.calBoost > cat  ) {
				//double Kp, double Ki, double Kd
				pid->SetTunings(cKp, cKi, cKd);
				aggressiveSettings = false;
			} else {
				pid->SetTunings(aKp, aKi, aKd);
				aggressiveSettings = true;
			}
			pid->Compute();
			boostOutput = pidBoostOutput;
		} else {
			//we're under the PID activation thresholds
			//set the map pwm value as base for the pid controller
			pidBoostOutput = req_Boost_PWM;
			//set the map pwm as output pwm
			boostOutput = req_Boost_PWM;
		}
	} else {
		//no PID
		boostOutput = req_Boost_PWM;
	}

	//overboost protection
	if ( data.calBoost > n75_max_boost ) {
		boostOutput = boostOutput * 0.75;
		pidBoostOutput = boostOutput;
	}
}

void BoostController2::computeSimple () {
	req_Boost_PWM = n75_pwm;
	req_Boost = n75_boost;

	if ( usePID ) {
		//give the PID the requested boost level
		pidBoostSetPoint = req_Boost;
		pid->SetTunings(aKp, aKi, aKd);
		pid->Compute();
		boostOutput = pidBoostOutput;
	} else {
		//no PID
		boostOutput = req_Boost_PWM;
	}

	//overboost protection
	if ( data.calBoost > n75_max_boost ) {
		boostOutput = boostOutput * 0.75;
		pidBoostOutput = boostOutput;
	}
}

void BoostController2::setN75Params (uint16_t *data) {
	aKp = fixedintb1002float ( *data );
	++data;
	aKi = fixedintb1002float ( *data );
	++data;
	aKd = fixedintb1002float ( *data );
	++data;
	cKp = fixedintb1002float ( *data );
	++data;
	cKi = fixedintb1002float ( *data );
	++data;
	cKd = fixedintb1002float ( *data );
	++data;
	apidActivationThresholdFactor = fixedintb1002float ( *data );
	++data;
	cpidActivationThresholdFactor = fixedintb1002float ( *data );
	++data;
	if ( ((uint8_t) (*data)) & 1 )
		usePID = true;
	else
		usePID = false;
}

