/*
    Copyright 2009-10 Stephan Martin, Dominik Gummel

    This file is part of Multidisplay.

    Multidisplay is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Multidisplay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Multidisplay.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MainController.h"
#include "BoostController2.h"

#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <wiring.h>
#include <EEPROM.h>
#include <WProgram.h>

#include <PID_v1.h>




MainController::MainController() {
}

void  MainController::myconstructor() {


	FlashETimeU = 0;
	FlashTimeU = 0;
	ScreenSave = 0;
	time = 0;


	DoCheck = 1;
	SerOut = SERIALOUT_ENABLED;

	pinMode (GREENLEDPIN, OUTPUT);
	pinMode (REDLEDPIN, OUTPUT);
	pinMode (ONBOARDBUTTONPIN, INPUT);
	digitalWrite(ONBOARDBUTTONPIN,HIGH);

	pinMode (NORDSCHLEIFENPIN, INPUT);
	digitalWrite( NORDSCHLEIFENPIN, HIGH); // turn on pullup resistors

	pinMode (N75PIN, OUTPUT);
	//lower the PWM Frequency
	setPwmFrequency(9, 64);

	Serial.begin(57600);

	Serial.println("BoostController  0.1!");

	//Turn On the Green LED:
	digitalWrite(GREENLEDPIN, HIGH);

#ifdef READFROMEEPROM
	readSettingsFromEeprom();
#endif


	//Init the Buttons:

	serialTime = millis();
}



void MainController::readSensors() {
	data.anaBoost = analogRead(BOOSTPIN);
	data.anaPoti = analogRead(POTIPIN);
	data.anaVoltage = analogRead(VOLTAGEPIN);
}

void MainController::anaConversion() {

	//Boost:
	data.calAbsoluteBoost = 5.0* ((float) data.anaBoost)/1024.0;

	//mpx4250
	//data.calAbsoluteBoost = (data.calAbsoluteBoost * 50 - 10)/100;

	// Freescale MPXA6400A
	// transfer function P = Vout/0.012105 + 3.477902
	data.calAbsoluteBoost = (data.calAbsoluteBoost/0.012105 + 3.477902) / 100;     	//makes 0-400kPa out of it

//	data.calBoostPSI = data.calBoostBar * BAR2PSI;
	data.calBoost = data.calAbsoluteBoost - data.boostAmbientPressureBar;			//apply the offset (ambient pressure)

	//Check if the Boost is a new Max Boost Event
	if( data.calBoost >= data.maxBoost ) {
		data.maxBoost = data.calBoost;
	}

	data.calPoti = map(data.anaPoti, 0, 1024, 0, 255);
}







void MainController::serialReceive() {

	int index=0;
	byte cmd = -1;
	while(Serial.available()&&index<25)
	{
		if(index==0) {
			cmd = Serial.read();
		} else
			srData.asBytes[index-1] = Serial.read();
		index++;
	}

	// if the information we got was in the correct format,
	// read it into the system
	switch (cmd) {
		case 1:
			if (index >= 4) {
				boostController.n75_pwm = srData.asBytes[0];
				boostController.n75_boost = fixedintb1002float( srData.asBytes[1] );
				boostController.n75_max_boost = fixedintb1002float( srData.asBytes[3] );
			}
			break;
		case 2:
			//TODO pid settings
			break;
		case 3:
			//TODO activate PID
			break;
		case 4:
			//TODO boost limit
			break;
		case 5:
			//TODO calibrate environment boost
			break;
		case 6:
			//TODO save settings 2 eeprom
			break;

	}

	Serial.flush();                         // * clear any random data from the serial buffer
}

//void MainController::saveSettings2Eeprom() {
//
//	EEPROM.write (EEPROM_N75_MANUALDUTY_NORMAL, boostController.n75_manual_normal);
//	EEPROM.write (EEPROM_N75_MANUALDUTY_RACE, boostController.n75_manual_race);
//}

void MainController::readSettingsFromEeprom() {
	long eval = EEPROMReadLong(EEPROM_AMBIENTPRESSURE);

	if ( eval < 0xFFFFFFFF ) {
		float ldt = eval /1000.0;      //gets the float back (thats accurate enough)
		if ( ldt > 0.0 && ldt < 1.2 )
			data.boostAmbientPressureBar = ldt;
	} else {
		data.boostAmbientPressureBar = 225;
	}
}


void MainController::serialPrintHeader() {
	//TODO
}

void MainController::serialSend() {

	Serial.print("\2");
	Serial.print("PID ");
	Serial.print(boostController.req_Boost_PWM);
	Serial.print(" ");
	Serial.print(boostController.req_Boost);
	Serial.print(" ");
	Serial.print(data.calBoost);
	Serial.print(" ");
	Serial.print(boostController.boostOutput);
	Serial.print(" ");
	if (boostController.pid->GetMode()==AUTOMATIC)
		Serial.print("A");
	else
		Serial.print("M");
	Serial.print(" ");
	Serial.print(data.anaVoltage);
	Serial.print(" ");
	Serial.print(boostController.n75_max_boost);

	Serial.print("\3");
	Serial.println();
}


void MainController::ChangeSerOut()
{
	//Switch from RAW to Cal and vise versa.
	switch(SerOut){
	case SERIALOUT_DISABLED:
		SerOut = SERIALOUT_ENABLED;
		break;
	case SERIALOUT_ENABLED:
		SerOut = SERIALOUT_DISABLED;
		break;
	default:
		SerOut = SERIALOUT_DISABLED;
		break;
	}
//	HeaderPrint();
}
//-------------------------------------------------------------------------------------------------------

void MainController::CalibrateLD()
{
	data.boostAmbientPressureBar = data.calAbsoluteBoost;
	//and saved:
	EEPROMWriteLong(EEPROM_AMBIENTPRESSURE,data.boostAmbientPressureBar*1000);    //writes the float as long, will do it.

	//The MaxLD will be reset!
	data.maxBoost = 0;

	Serial.println(" ");
	Serial.print("CalBoost: ");
	Serial.println(data.boostAmbientPressureBar);
}



//----------------------------------------------------------------------------------------------------
//This checks certain values if they exceed limits or not, if so an alarm is triggered
void MainController::CheckLimits()
{
	uint8_t FlashTrigger=0;

	//Set The new Flash Timeout
	if(FlashTrigger>0)
	{
		FlashETimeU = millis() + FLASH_E_TIME;
		FlashTrigger = 0;
	}


	//And Flash!
	if(FlashETimeU>=millis())                     //Check if the Flash should be shown
	{
		if(FlashTimeU<=millis())                  //Check if the Flash is about to be changed
		{

			FlashTimeU = millis() + FLASH_TIME;      //And save the Next Changetime
		}
	}

}

//----------------------------------------------------------------------------------------------------







void MainController::mainLoop() {
	//Current Time:
	time = millis();


	readSensors();

	anaConversion();

	//Check the Button
	if(digitalRead( ONBOARDBUTTONPIN ) == LOW) {
		CalibrateLD();
	}


#if FREEMEM
	//Free Mem Debug:
	Serial.print("FM ");
	Serial.println( freeMem());
#endif


	//internal pull-up active
	//switch connects pin to groud
	boostController.usePID = digitalRead(NORDSCHLEIFENPIN);
	if ( boostController.usePID ) {
		digitalWrite(REDLEDPIN, HIGH);
		boostController.n75_pwm = data.calPoti;
		boostController.n75_boost = (double) map(data.anaPoti, 0, 1024, 0, 200) / 100.0;
		boostController.n75_max_boost = boostController.n75_boost + 0.1;
	} else {
		boostController.n75_pwm = data.calPoti;
	}

	boostController.computeSimple();
	analogWrite(N75PIN, (int) boostController.boostOutput);
//	analogWrite(FREEPWM2, (int) boostController.boostOutput);

	if ( millis() > serialTime ) {
		serialReceive();
		//Print it:
		serialSend();
		serialTime += SERIALFREQ;
	}
}

void MainController::setPwmFrequency(int pin, int divisor) {
  byte mode;
  if(pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if(pin == 5 || pin == 6) {
      (TCCR0B = TCCR0B & 0b11111000) | mode;
    } else {
      (TCCR1B = TCCR1B & 0b11111000) | mode;
    }
  } else if(pin == 3 || pin == 11) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x7; break;
      default: return;
    }
    (TCCR2B = TCCR2B & 0b11111000) | mode;
  }
}
